<?php
// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
    die();
}

/**
 * SEO Cleaner Admin Settings Class
 */
class SEO_Cleaner_Admin {

    /**
     * List of settings fields
     */
    public $fields = array(
        'head' => array(
            'tab_title'   => 'Head',
            'title'       => 'Head Cleanup',
            'description' => 'Remove redundant links, scripts and styles from head section.',
            'fields'      => array(
                array(
                    'name'        => 'remove_wp_version',
                    'title'       => 'Remove WordPress Generator Version',
                    'type'        => 'checkbox',
                    'description' => 'Remove "<meta name=\'generator\'..." from head section',
                    'notes'       => 'This meta tag just shows your WordPress version. You can remove it for security reasons and keep your head section clean.',
                ),
                array(
                    'name'        => 'remove_wlwmanifest',
                    'title'       => 'Remove WLW Manifest Link',
                    'type'        => 'checkbox',
                    'description' => 'Remove "<link rel=\'wlwmanifest\'..." from head section',
                    'notes'       => 'WLW Manifest Link adds support for Windows Live Writer app. If you are not using this app you can remove this link.',
                ),
                array(
                    'name'        => 'remove_rsd',
                    'title'       => 'Remove RSD Link (EditURI Link)',
                    'type'        => 'checkbox',
                    'description' => 'Remove "<link rel=\'EditURI\'..." from head section',
                    'notes'       => 'RSD (Real Simple Discovery) is the mechanism used by XML-RPC clients. Also this mechanism can be used by spammers. If do not use this clients with your site, you can remove it.',
                ),
                array(
                    'name'        => 'remove_shortlink',
                    'title'       => 'Remove Shortlink',
                    'type'        => 'checkbox',
                    'description' => 'Remove "<link rel=\'shortlink\'..." from head section',
                    'notes'       => 'This meta tag just shows short link (not canonical link) to your posts. You can remove it.',
                ),
                array(
                    'name'        => 'remove_prev_next',
                    'title'       => 'Remove Prev/Next Links',
                    'type'        => 'checkbox',
                    'description' => 'Remove "<link rel=\'prev\'..." and "<link rel=\'next\'..." from head section',
                    'notes'       => 'This is links to the next and previous post that actually not used. You can remove it and keep your head section clean.',
                ),
                array(
                    'name'        => 'remove_recentcomments',
                    'title'       => 'Remove Hardcoded Styles for Recent Comments',
                    'type'        => 'checkbox',
                    'description' => 'Remove styles for ".recentcomments a" from head section',
                    'notes'       => 'This is hard coded css for recent comments widget. You can remove it and keep your head section clean.',
                ),
                array(
                    'name'        => 'remove_emoji',
                    'title'       => 'Remove Emoji Styles and Scripts',
                    'type'        => 'checkbox',
                    'description' => 'Remove Emoji styles and scripts from head section',
                    'notes'       => 'If you are not using emoji smiles on your site you can disable this feature and keep your head section clean.',
                ),
                array(
                    'name'        => 'remove_embed',
                    'title'       => 'Remove WP-Embed Script',
                    'type'        => 'checkbox',
                    'description' => 'Remove wp-embed.min.js script',
                    'notes'       => 'You may want to disable this feature to have fewer HTTP requests.',
                ),
                array(
                    'name'        => 'remove_migrate',
                    'title'       => 'Remove jQuery-Migrate Script',
                    'type'        => 'checkbox',
                    'description' => 'Remove jquery-migrate.min.js script',
                    'notes'       => 'You may want to disable this feature to have fewer HTTP requests.',
                ),
                array(
                    'name'        => 'remove_dns_prefetch',
                    'title'       => 'Remove DNS prefetch',
                    'type'        => 'checkbox',
                    'description' => 'Remove &lt;link rel=&quot;dns-prefetch&quot; href=&quot;http://www.example.com/&quot;&gt; from head section',
                    'notes'       => 'If all of your resources has the same domain yo can disable forcing DNS lookup of specific hostnames.',
                ),
            ),
        ),
        'feed' => array(
            'tab_title'   => 'Feed',
            'title'       => 'Feed Cleanup',
            'description' => 'Cleanup your head section from feed links and redirect them to the homepage if needed.',
            'fields'      => array(
                array(
                    'name'        => 'remove_feed',
                    'title'       => 'Remove Feed Links',
                    'type'        => 'checkbox',
                    'description' => 'Remove all feed links from head section',
                    'notes'       => 'This option does not disable feed functionality, just cleans head section.',
                ),
                array(
                    'name'        => 'redirect_feed',
                    'title'       => 'Totally Disable Feeds',
                    'type'        => 'checkbox',
                    'description' => 'Disable feeds feature by redirection to the homepage (also removes feed links from head section)',
                    'notes'       => 'This option totally disables feed functionality. If you are using feeds on your site (e.g. FeedBurner) do not disable feeds.',
                    'noimage'     => true
                ),
            ),
        ),
        'http' => array(
            'tab_title'   => 'HTTP Response',
            'title'       => 'HTTP Response Cleanup',
            'description' => 'Cleanup your server response HTTP headers.',
            'fields'      => array(
                array(
                    'name'        => 'remove_server_shortlink',
                    'title'       => 'Remove Shortlink from HTTP Headers',
                    'type'        => 'checkbox',
                    'description' => 'Remove "Link:<...>; rel=shortlink" from server response HTTP headers',
                    'notes'       => 'This response header contains link to posts short URLs. This information is not used anywhere and you can remove this.',
                ),
                array(
                    'name'        => 'remove_pingback',
                    'title'       => 'Remove X-Pingback from HTTP Headers',
                    'type'        => 'checkbox',
                    'description' => 'Remove "X-Pingback:..." from server response HTTP headers (works only with PHP 5.3 and up)',
                    'notes'       => 'This response header contains link to your pingback file. This information can be used by spammers and you can remove it.',
                ),
                array(
                    'name'        => 'remove_powered',
                    'title'       => 'Remove X-Powered-By from HTTP Headers',
                    'type'        => 'checkbox',
                    'description' => '——W—P—L—O—C—K—E—R—.—C—O—M—— Remove "X-Powered-By:..." from server response HTTP headers (works only with PHP 5.3 and up)',
                    'notes'       => 'This response header contains information about PHP version on your server. This information is not used anywhere and you can remove this.',
                ),
            ),
        ),
        'api' => array(
            'tab_title'   => 'WP JSON API',
            'title'       => 'WP JSON API Cleanup',
            'description' => 'Cleanup your site from WP JSON API features.',
            'fields'      => array(
                array(
                    'name'        => 'remove_api_head',
                    'title'       => 'Remove WP API Links and Scripts',
                    'type'        => 'checkbox',
                    'description' => 'Remove all WP JSON API links and scripts from head section',
                    'notes'       => 'This option does not disable WP API, just cleans head section from these links.',
                ),
                array(
                    'name'        => 'remove_api_server',
                    'title'       => 'Remove WP API Link from HTTP Headers',
                    'type'        => 'checkbox',
                    'description' => 'Remove "Link:<...>; rel=https://api.w.org/" from server response HTTP headers',
                    'notes'       => 'This option does not disable WP API, just cleans HTTP headers.',
                ),
                array(
                    'name'        => 'disable_api',
                    'title'       => 'Totally Disable WP API Feature',
                    'type'        => 'checkbox',
                    'description' => 'Disable WP JSON API functionality on your site',
                    'notes'       => 'WP API is used by external apps to get data from your site. If you are not using this feature you can disable this.',
                    'noimage'     => true
                ),
            ),
        ),
        'comments' => array(
            'tab_title'   => 'Comments',
            'title'       => 'Comments Cleanup',
            'description' => 'Cleanup your comment form and comments template.',
            'fields'      => array(
                array(
                    'name'        => 'remove_comments_url',
                    'title'       => 'Remove Website Field',
                    'type'        => 'checkbox',
                    'description' => 'Remove website (URL) field from comment form template',
                    'notes'       => 'Website field in comments is a popular way to leave spam link. You can avoid such kind of spam.',
                ),
                array(
                    'name'        => 'remove_comments_notes',
                    'title'       => 'Remove Notes Before Comment Form',
                    'type'        => 'checkbox',
                    'description' => 'Remove "Your email address will not be published..." from comment form template',
                    'notes'       => 'This note is used on many websites. You can remove it as not unique content.',
                ),
                array(
                    'name'        => 'remove_comments_author_link',
                    'title'       => 'Remove Comment Author Link',
                    'type'        => 'checkbox',
                    'description' => 'Remove website link from comment author name',
                    'notes'       => 'By default comment author name is linked to its website. You can avoid this.',
                ),
                array(
                    'name'        => 'remove_comments_autolinking',
                    'title'       => 'Disable Auto Linking',
                    'type'        => 'checkbox',
                    'description' => 'Disable auto linking in comment content (URLs will stay as plain text as were posted)',
                    'notes'       => 'When user adds URL as plain text to the comment WordPress automatically converts it to the link tag. You can disable this feature.',
                ),
            ),
        ),
        'archives' => array(
            'tab_title'   => 'Archives',
            'title'       => 'Archives Redirect',
            'description' => 'Redirect unused archives to the homepage to avoid indexing these pages and content duplication.',
            'fields'      => array(
                array(
                    'name'        => 'remove_archives_date',
                    'title'       => 'Disable Date Archives',
                    'type'        => 'checkbox',
                    'description' => 'Redirect date archives to the homepage',
                    'notes'       => 'If you are not using date archives on your site you can disable them.',
                    'noimage'     => true
                ),
                array(
                    'name'        => 'remove_archives_author',
                    'title'       => 'Disable Author Archives',
                    'type'        => 'checkbox',
                    'description' => 'Redirect author archives to the homepage',
                    'notes'       => 'If you have only 1 author on your site or you are not planning to use this archives you can disable them.',
                    'noimage'     => true
                ),
                array(
                    'name'        => 'remove_archives_tag',
                    'title'       => 'Disable Tag Archives',
                    'type'        => 'checkbox',
                    'description' => 'Redirect tag archives to the homepage',
                    'notes'       => 'If you are not using tags on your site you can disable these archives.',
                    'noimage'     => true
                ),
                array(
                    'name'        => 'remove_archives_category',
                    'title'       => 'Disable Category Archives',
                    'type'        => 'checkbox',
                    'description' => 'Redirect category archives to the homepage',
                    'notes'       => 'If you are not using categories on your site you can disable these archives.',
                    'noimage'     => true
                ),
                array(
                    'name'        => 'remove_archives_postformat',
                    'title'       => 'Disable Post Format Archives',
                    'type'        => 'checkbox',
                    'description' => 'Redirect post format archives to the homepage',
                    'notes'       => 'If you are not using post formats on your site you can disable these archives.',
                    'noimage'     => true
                ),
                array(
                    'name'        => 'remove_archives_search',
                    'title'       => 'Disable Search',
                    'type'        => 'checkbox',
                    'description' => 'Redirect search listing to the homepage',
	                'notes'       => 'If you do not use search on your site you can disable search results page.',
                    'noimage'     => true
                ),
            ),
        ),
        'attachment' => array(
            'tab_title'   => 'Attachments',
            'title'       => 'Attachments Redirect',
            'description' => 'Redirect attachment single page URLs to parent post URL to avoid indexing these pages.',
            'fields'      => array(
                array(
                    'name'        => 'remove_attachment',
                    'title'       => 'Disable Attachments',
                    'type'        => 'checkbox',
                    'description' => 'Redirect attachment pages archives to parent post URL',
                    'notes'       => 'Every image or other file attached to post has it\'s own URL and sometimes it can hurt your SEO if these URLs will be indexed by search engines.',
                    'noimage'     => true
                ),
            ),
        ),
    );

    /**
     * Hook into the appropriate actions when the class is constructed
     */
    public function __construct() {

        $this->fields = apply_filters( 'seo_cleaner_admin_fields', $this->fields );

        add_action( 'admin_menu',  array( $this, 'add_admin_menu' ) );
        add_action( 'admin_init', array( $this, 'settings_init' ) );
        add_filter( 'plugin_action_links_'. SEOCL_BASENAME, array( $this, 'add_action_links' ) );
        add_action( 'admin_enqueue_scripts', array( $this, 'add_scripts' ) );

    }

    /**
     * Add settings link
     */
    function add_action_links( $links ) {

        $settings_link = array(
            '<a href="' . admin_url( 'options-general.php?page=seocl' ) . '">' . __( 'Settings', 'seocl' ) . '</a>',
        );

        return array_merge( $links, $settings_link );

    }

    /**
     * Add admin css and js files
     */
    public function add_scripts( $hook ){

        if( $hook == 'settings_page_seocl' ) {
            wp_enqueue_style( 'seocl-css', plugins_url( 'css/admin.css', __FILE__ ) );
            wp_enqueue_script( 'seocl-js', plugins_url( 'js/admin.js', __FILE__ ), array( 'jquery', 'jquery-ui-tooltip' ) );
        }

    }

    /**
     * Add menu page for plugin settings
     */
    public function add_admin_menu() {

        add_submenu_page(
            'options-general.php',
            'SEO Cleaner',
            'SEO Cleaner',
            'manage_options',
            'seocl',
            array( $this, 'seocl_options_page' )
        );

    }

    /**
     * Retrieve current tab
     */
    public function get_current_tab() {

        if( isset( $_GET['tab'] ) ) {

            $_tab = (string)$_GET['tab'];

            if( ! empty( $this->fields[ $_tab ] ) )
                return $_tab;

        }

        return 'head';

    }

    /**
     * Register plugin settings
     */
    public function settings_init() {

        $_tab = $this->get_current_tab();

        foreach ( $this->fields as $key => $value ){
            register_setting( 'seocl_settings_' . $key, 'seocl_settings_' . $key );
        }

        if( ! empty( $this->fields[ $_tab ] ) ) {

            add_settings_section(
                'seocl_settings_' . $_tab,
                __( $this->fields[ $_tab ]['title'], 'seocl' ),
                array( $this, 'seocl_settings_section_callback' ),
                'seocl_settings_' . $_tab
            );

            foreach ( $this->fields[ $_tab ]['fields'] as $field ) {

                add_settings_field(
                    $field['name'],
                    __( $field['title'], 'seocl' ),
                    array( $this, 'render_field' ),
                    'seocl_settings_' . $_tab,
                    'seocl_settings_' . $_tab,
                    $field
                );

            }

        }

    }

    /**
     * Callback for setting tab description
     */
    public function seocl_settings_section_callback() {

        $_tab = $this->get_current_tab();

        if( ! empty( $this->fields[ $_tab ] ) )
            _e( $this->fields[ $_tab ]['description'], 'seocl' );

    }

    /**
     * Settings page output
     */
    public function seocl_options_page() {

        $_tab = $this->get_current_tab();
        ?>
        <form class="seocl" action='options.php' method='post'>

            <h1><?php _e( 'SEO Cleaner', 'seocl' ); ?></h1>

            <div class="nav-tab-wrapper">
                <?php foreach ( $this->fields as $name => $label ): ?>
                    <a href="<?php echo  admin_url( 'options-general.php?page=seocl&tab=' . $name ) ?>" class="nav-tab <?php echo ( $this->get_current_tab() == $name ? 'nav-tab-active' : '' ) ?>">
                        <?php echo $label['tab_title'] ?>
                    </a>
                <?php endforeach; ?>
            </div>

            <?php
            settings_fields( 'seocl_settings_' . $_tab );
            do_settings_sections( 'seocl_settings_' . $_tab );
            submit_button();
            ?>

        </form>
        <?php

    }

    /**
     * Generate field
     */
    public function render_field( $args ) {

        $_tab = $this->get_current_tab();
        $checked_value = isset( SEO_Cleaner::$settings[ $_tab ][ $args['name'] ] ) ? SEO_Cleaner::$settings[ $_tab ][ $args['name'] ] : 0;
        ?>
        <label>
            <input
                type="checkbox"
                name="seocl_settings_<?php echo $_tab . '[' . esc_attr( $args['name'] ) . ']' ?>"
                value="1"
                <?php checked( $checked_value, 1 ) ?>
            >
            <?php esc_html_e( $args['description'], 'seocl' ) ?>
        </label>

        <?php if( empty( $args['noimage'] ) ) { ?>
            <span class="seocl-tooltip hide-if-no-js" title="" data-img="<?php echo plugins_url( 'images/' . $args['name'] . '.png', __FILE__ ) ?>">?</span>
        <?php } ?>

        <?php if( ! empty( $args['notes'] ) ) { ?>
            <p class="description"><?php esc_html_e( $args['notes'], 'seocl' ) ?></p>
        <?php } ?>

        <?php

    }

}